#define _CRT_SECURE_NO_DEPRECATE
#pragma comment (linker, "/STACK:30000000")

#include <cmath>
#include <string>
#include <vector>
#include <algorithm>
#include <iostream>
#include <cassert>
#include <cstring>


#ifndef __random_h__
#define __random_h__

#include <cmath>
#include <cstdio>
#include <cstdlib>

const int MBIG = 1000000000, MSEED = 161803398;
const double FAC = 1.0 / MBIG;

int inext, inextp;
int ra [56];

int nextrand (void)
{
  int j;
  if (++inext == 56) inext = 1;
  if (++inextp == 56) inextp = 1;
  j = ra[inext] - ra[inextp];
  if (j < 0) j += MBIG;
  ra[inext] = j;
  return j;
}

bool initrand (int seed)
{
  int i, j, k, l;
  l = MSEED - seed;
  if (l < 0) l = -l;
  l %= MBIG;
  ra[55] = l;
  k = 1;
  for (i = 1, j = 0; i <= 54; i++)
  {
   j += 21;
   if (j > 55) j -= 55;
   ra[j] = k;
   k = l - k;
   if (k < 0) k += MBIG;
   l = ra[j];
  }
  for (k = 1; k <= 4; k++)
   for (i = 1, j = 31; i <= 55; i++, j >= 55 ? j = 1 : j++)
   {
    ra[i] -= ra[j];
    if (ra[i] < 0) ra[i] += MBIG;
   }
  inext = 0;
  inextp = 31;
  return true;
}

double rndvalue (void)
{
  return nextrand () * FAC;
}

int rndvalue (int k)
{
  return (int) floor (rndvalue () * k);
}

long long Time()
{
#ifdef __GNUC__
  long long res;
  asm volatile ("rdtsc" : "=A" (res));
  return res;
#else
  int lo, h;
  __asm{
    rdtsc
    mov lo, eax
    mov h, edx
  }
  return (((long long)h) << 32ll) | lo;
#endif
}

#define R(A, B) ((A) + rndvalue((B) - (A) + 1))

#endif // __random_h__



using namespace std;

#define EPS 1E-8
const int INF = (int)1E+9;

#define forn(i, n) for (int i = 0; i < (int)(n); ++i)
#define forv(i, a) for (int i = 0; i < int(a.size()); i++)
#define all(a) a.begin(), a.end()
#define norm(a) sort(all(a)); a.erase(unique(all(a)), a.end());

FILE *f = 0;
int tst = 9;

void close() {
    if (f != 0) {
        fclose(f);
        f = 0;
    }
}

FILE *open() {
    close();
    char fname[10];
    sprintf(fname, "%.3d", ++tst);
    cerr << tst << endl;
    return f = fopen(fname, "wt");
}

int random(int value)
{
    return nextrand() % value;
}

void sample() {
    open();
    fprintf(f, "4 4 2\n1 3\n1 2\n2 3\n3 4\n4 1\n");
    open();
    fprintf(f, "5 5 3\n5 1 3\n1 2\n2 3\n5 2\n3 4\n4 1\n");
}

void hand() {
    open();
    fprintf(f, "5 5 3\n5 1 3\n1 2\n2 3\n5 3\n3 4\n4 1\n");
    open();
    fprintf(f, "6 6 2\n1 6\n1 2\n1 3\n2 4\n3 4\n4 5\n5 6\n");
}

const int MAXN = 100000;
int n, m;
vector<int> g[MAXN];
int t[MAXN];
int dst[MAXN];
int q[MAXN];
int hd, tl;

int calcEdges() {
    m = 0;
    forn(i, n) {
        norm(g[i]);
        m += (int)g[i].size();
    }
    m /= 2;
	return m;
}

void out() {
    open();
	calcEdges();
    vector<int> p(n);
    forn(i, n) p[i] = i + 1;
    random_shuffle(all(p));
    int kt = 0;
    forn(i, n) kt += t[i] == 1;
    fprintf(f, "%d %d %d\n", n, m, kt);
    bool first = true;
    forn(i, n) if (t[i] == 1) {
        if (!first) {
            fprintf(f, " ");
        }
        first = false;
        fprintf(f, "%d", p[i]);
    }
    fprintf(f, "\n");
    forn(i, n) {
        forv(j, g[i]) {
            if (g[i][j] > i) {
                fprintf(f, "%d %d\n", p[i], p[g[i][j]]);                
            }
        }
    }
}

void genRandomGraph(int n, int m) {
    ::n = n;
    ::m = m;
    forn(i, n) g[i].clear();
    forn(i, n - 1) {
        int u = i + 1, v = random(i + 1);
        g[u].push_back(v);
        g[v].push_back(u);
    }
    forn(i, m - n + 1) {
        int u = random(n), v = random(n);
        while (u == v) {
            u = random(n), v = random(n);
        }
        g[u].push_back(v);
        g[v].push_back(u);
    }
}

void colorTwo(int v, int c) {
    if (t[v] != -1) return ;
    t[v] = c;
    forv(i, g[v]) colorTwo(g[v][i], 1 - c);
}

void simpleRandom(int n, int m) {
    genRandomGraph(n, m);
    forn(i,n) t[i] = random(2);
    out();
}

void biparate(int n, int m) {
    ::n = n;
    ::m = m;
    forn(i, n) g[i].clear();
    forn(i, n - 1) {
        int u = i + 1, v = random(i + 1);
        g[u].push_back(v);
        g[v].push_back(u);
    }
    memset(t, 255, sizeof(t[0]) * n);
    colorTwo(0, 0);
    vector<int> qt, qnt;
    forn(i, n) if (t[i] == 1) {
        qt.push_back(i);
    } else {
        qnt.push_back(i);
    }
    forn(i, m - n + 1) {
        int u = qt[random(qt.size())], v = qnt[random(qnt.size())];
        g[u].push_back(v);
        g[v].push_back(u);
    }
    out();
}

void cycle(int n, int d) {
    ::n = n;
    ::m = n;
    forn(i, n) g[i].clear();
    forn(i, n) {
        g[i].push_back((i + 1) % n);
        g[(i + 1) % n].push_back(i);
    }
    forn(i, n) {
        t[i] = i % d == 0;
    }
    out();
}

void add_line_to_vertex(int& v, int c, int len) {
    t[v] = 1;
    forn(j, len - 1) {
        g[v].push_back(v + 1);
        g[v + 1].push_back(v);
        ++v;
    }
    g[v].push_back(c);
    g[c].push_back(v);
}

void add_g_to_vertex(int& v, int c, int kv, int dst)
{
    vector<int> sloys(dst + 1, 1);
    forn(i, kv - dst) {
        int t = random(dst - 1) + 1;
        sloys[t]++;
    }
    t[v] = 1;
    int cs = v;
    forn(i, dst) {
        int ns = i + 1 < dst ? cs + sloys[i] : c;
        forn(j, sloys[i]) {
            int x = cs + j;
            int y = ns + random(sloys[i + 1]);
            g[x].push_back(y);
            g[y].push_back(x);
        }
        int kit = random(max(sloys[i], sloys[i + 1]));
        forn(j, kit) {
            int x = cs + random(sloys[i]);
            int y = ns + random(sloys[i + 1]);
            g[x].push_back(y);
            g[y].push_back(x);
        }
        cs = ns;
    }
    v += kv - 1;
}

void center_line(int n, int k) {
    assert((n - 1) % k == 0);
    ::n = n;
    ::m = n - 1;
    forn(i, n) g[i].clear();
    int c = 0, v = 0;
    memset(t, 0, sizeof(t[0]) * n);
    forn(i, k) {
        ++v;
        add_line_to_vertex(v, c, (n - 1) / k);
		assert(v < n);
    }
	assert(v == n - 1);
    out();
}

void center_line_pol(int n, int k) {
    assert((n - k) % k == 0);
    ::n = n;
    ::m = n;
    forn(i, n) g[i].clear();
    int v = k - 1;
    memset(t, 0, sizeof(t[0]) * n);
    forn(i, k) {
        ++v;
        add_line_to_vertex(v, i, (n - k) / k);
		assert(v < n);
    }
    forn(i, k) {
        g[i].push_back((i + 1) % k);
        g[(i + 1) % k].push_back(i);
    }
	assert(v == n - 1);
    out();
}

void center_line_double_pol(int n, int k) {
    assert((n - 2 * k) % k == 0);
    ::n = n;
    ::m = n;
    forn(i, n) g[i].clear();
    int v = 2 * k - 1;
    memset(t, 0, sizeof(t[0]) * n);
    forn(i, k) {
        ++v;
        add_line_to_vertex(v, i * 2, (n - 2 * k) / k);
		assert(v < n);
    }
    k *= 2;
    forn(i, k) {
        g[i].push_back((i + 1) % k);
        g[(i + 1) % k].push_back(i);
    }
	assert(v == n - 1);
    out();
}

void center_g_pol(int n, int k) {
    assert((n - k) % k == 0);
    ::n = n;
    ::m = n;
    forn(i, n) g[i].clear();
    int v = k - 1;
    memset(t, 0, sizeof(t[0]) * n);
    int dst = min(max(random((n - 2 * k) / k), 2), (n - 2*k) / k);
	dst = max(dst, 2);
    forn(i, k) {
        ++v;
        add_g_to_vertex(v, i, (n - k) / k, dst);
		assert(v < n);
    }
    forn(i, k) {
        g[i].push_back((i + 1) % k);
        g[(i + 1) % k].push_back(i);
    }
	assert(v == n - 1);
    out();
}

void palma(int n, int m, int add, int k) {
    genRandomGraph(n, m - k * add);
    ::n += k * add;
	forn(i, k * add) g[n + i].clear();
    int v = random(n);
    forn(i, add) {
        g[n + i].push_back(v);
        g[v].push_back(n + i);
    }
    memset(t, 0, sizeof(t[0]) * (n + k * add));
    forn(q, k - 1) {
        forn(i, add) {
            g[n + (q + 1) * add + i].push_back(n + q * add + i);
            g[n + q * add + i].push_back(n + (q + 1) * add + i);
        }
    }
    forn(i, add) {
        t[n + (k - 1) * add + i] = 1;
    }
    out();
}

void center_g_double_pol(int n, int k) {
    assert((n - 2 * k) % k == 0);
    ::n = n;
    ::m = n;
    forn(i, n) g[i].clear();
    int v = 2 * k - 1;
    memset(t, 0, sizeof(t[0]) * n);
    int dst = min(max(random((n - 2*k) / k), 2), (n - 2*k) / k);
	dst = max(dst, 2);
    forn(i, k) {
        ++v;
        add_g_to_vertex(v, i * 2, (n - 2 * k) / k, dst);
		assert(v < n);
    }
    k *= 2;
    forn(i, k) {
        g[i].push_back((i + 1) % k);
        g[(i + 1) % k].push_back(i);
    }
	assert(v == n - 1);
    out();
}

void solve() {
    cerr << "simpleRandom" << endl;
    simpleRandom(10, 50);
    simpleRandom(1000, 50000);
    simpleRandom(10000, 500000);
    simpleRandom(50000, 500000);
    simpleRandom(100000, 500000);

    cerr << "cycle" << endl;
    for(int n = 1000; n <= 100000; n *= 10) {
        for(int dst = 5; dst < n / 2; dst *= 10) {
            cycle(n, dst);
        }
    }

}

void readdata() {
}

int main(int, char*[]) 
{
    initrand(3821782);
    readdata();
    solve();
    return 0;
}

